
/*     *******************************************************************************
	XVisual GUI Development System for Linux/X-Window
	Copyright (C) 1996 Steve Carrie.

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
        ******************************************************************************* */

/* =============================================
	xvis user interface builder for Athena widgets using
	xv class library.

	Global Files

	V1.0 BETA  - last edited 01/07/96
    ============================================= */

#ifndef _Vxvis_h
#define _Vxvis_h

#include <X11/xv/xv.h>
#include <bitmaps/xvis.xpm>

// ---------------------------------------------
// Global Types & Structures

#define CALLMODE	0
#define EVENTMODE	1

#define MAXOBJECTS 200
#define MAXTOOLS 19
#define NUMEVENTS 16
#define MAXPROJECTFILES 20
#define MAXPULLDOWNS	20
#define MAXMENUENTRY	20
#define MAXEDITWINDOWS 10
#define MAXEDITBUFFERSIZE	32767
#define MAXFILELIST	40

#define BCOLOUR 1
#define FCOLOUR 2
#define TSCOLOUR 3
#define BSCOLOUR 4

enum objectType { 	XVTimer, XVList, XVButton, XVRepeater, XVBitmap, XVLabel, 
				XVDropDown, XVEditField, XVBox, XVToggle, XVForm, XVVScroll,
				XVHScroll, XVPanner, XVFileList, XVDirList, XVWindow, XVPointer, 
				XVNone, XVAll };

enum resourceType { 	XVVoid, XVPixel, XVDimension, XVInt, XVBoolean, XVPosition,
					XVString, XVName, XVCursor, XVWinName, XVWinIcon,
					XVWinPositionX, XVWinPositionY, XVWinSize, XVFont,
					XVPath, XVBitmapFile, XVObject, XVTime, XVLocBoolean,
					XVWrap, XVvScroll, XVhScroll, XVResize, XVEditmode,
					XVShapeStyle, XVJustify };

enum eventType {	XVButtonPress, XVButtonRelease, XVEnterNotify, XVLeaveNotify,
				XVFocusIn, XVFocusOut, XVExpose, XVKeyPress, XVKeyRelease,
				XVStructureNotify, XVMap, XVUnmap, XVPointerMotion, 
				XVLeftButtonMotion, XVMidButtonMotion, XVRightButtonMotion };

enum callType {	XVNoCall, XVCallback, XVEvent, XVTimeout };

struct filelist {
	char filename[80];
	char stubname[80];
	int source;
	int sourcetype;
};

struct rgb {
	char red;
	char blue;
	char green;
	char name[25];
};

struct event_data {	
	char *eventName;
	unsigned long eventbits;
	char *eventMask;
	char *eventStruct;
};

#define XVMENUVOID	0
#define XVMENUEND	1
#define XVMENUMAIN	2
#define XVMENUSUB	3
#define XVMENUSEP	4

struct editcontrol {
	Boolean in_use;
	char name[80];
	xvEditWindow *editor;
	char *edit_buffer;
	int mode;
	int ref;
	int object;
	char client[10];
};

struct modulecontrol {
	Boolean in_use;
	char name[80];
	char path[220];
	xvEditWindow *editor;
	char *edit_buffer;
	char client[10];
	Boolean active;
	Boolean changed;
	Boolean first;
};

struct menuentry {
	int type;
	char name[40];
	char caption[40];
	char callback[40];
	int listpoint;
	char *codebuffer;
};

struct object_listdata {
	char listitem[80];
	int object_ref;
};

struct loadref {
	int reference;
	char index_name[80];
	char window_name[80];
	char obref1[80];
	char obref2[80];
	char obref3[80];
};
	
struct resource_store {
	char *r_restype;
	char r_value[120];
	resourceType r_type;
	union {
		Dimension r_dimension;
		int r_int;
		Boolean r_boolean;
		Position r_position;
		String r_string;
		String r_cursor;
	};
};

struct objectlist {
	char name[40];
	char obj_cursor[40];
	char obj_font[120];
	char obj_path[180];
	char obj_icon[180];
	char obj_bitref[50];
	char leftbitmap[180];
	char obj_leftbitref[50];
	char obj_menu_font[120];
	char background_colour[25];
	char foreground_colour[25];
	char border_colour[25];
	char topshadow_colour[25];
	char bottomshadow_colour[25];
	char *callbackcode[3];
	char *eventcode[16];
	objectType object_type;
	objectType parent_type;
	int edit_mode;
	int resize_mode;
	int vscroll_mode;
	int hscroll_mode;
	int wrap_mode;
	int shape;
	int justify;
	int object_number;
	int object_index;
	int parent_index;
	int parent_window;
	int num_children;
	int object_ref1;
	int object_ref2;
	int object_ref3;
	int object_time;
	int totalmenu;
	int nummenu;
	int numpulldown[MAXPULLDOWNS];
	Boolean autowrap;
	Boolean echo;
	Boolean ctrl;
	Boolean enabled;
	Boolean visible;
	Boolean changed;
	Boolean fixed_size;
	Boolean hasmenu;
	Boolean iconic;
	Boolean open_at_startup;
	Boolean has_bitmap;
	Boolean has_leftbitmap;
	unsigned long events;
	Position winxpos;
	Position winypos;
	Dimension winwidth;
	Dimension winheight;
	Widget object_widget;
	xvBitmap *object_bitmap;
	xvBitmap *leftbit;
	xvFont *object_font;
	xvMenuBar *menubar;
	xvPulldownMenu *pulldown[MAXMENUENTRY];
	char menu_background[25];
	char menu_foreground[25];
	struct menuentry *menuinfo;
	union {
		xvCore *pcore;
		xvTimer *ptimer;
		xvVisualList *plist;
		xvButton *pbutton;
		xvRepeater *prepeater;
		xvBitmap *pbitmap;
		xvLabel *plabel;
		xvDropDown *pdropdown;
		xvEditField *peditfield;
		xvBox *pbox;
		xvToggle *ptoggle;
		xvVisualForm *pform;
		xvScrollbar *pvscroll;
		xvScrollbar *phscroll;
		xvPanner *ppanner;
		xvWindow *pwindow;
		xvFileList *pfilelist;
		xvFileList *pdirlist;		
	};
};

// ---------------------------------------------
// External Global working data

extern struct objectlist objects[];
extern struct loadref loading_ref[];
extern int nobject;
extern Boolean uses3D[];
extern char *objectName[];
extern char *className[];
extern int objectInstance[];
extern objectType objectList[];
extern char *listvalues[];
extern eventType eventList[];
extern struct event_data eventinfo[];
extern struct rgb colour[];
extern char *colournamelist[];
extern int numcolours;
extern struct editcontrol editor[];
extern struct modulecontrol module[];
extern struct filelist cc_files[];
extern struct filelist h_files[];
extern int numccfiles;
extern int numhfiles;
extern char *wrapinfo[], *scrollinfo[], *resizeinfo[];
extern char *wrapdata[];
extern char *scrolldata[];
extern char *resizedata[];
extern char *editmodeinfo[];
extern char *editmodedata[];
extern char *buttonshapeinfo[];
extern char *buttonshapedata[];
extern int buttonshapecmd[];
extern char *justifyinfo[];
extern char *justifydata[];
extern XtJustify justifycmd[];

// ---------------------------------------------
// Global info

extern int screenWidth, screenHeight;
extern Pixel blackPixel, whitePixel;
extern int screenDepth;
extern unsigned long screenMask;

// ---------------------------------------------
// External Global Class declarations.

extern xvContext xvisContext;

extern xvFont cursorfont;
extern xvMsgBox alert;

extern xvWindow xvisSelectFile;
	extern xvFileDialog xvisSelFile;

extern xvWindow xvisControl;
	extern xvMenuBar xvisControlMenu;
	extern xvPulldownMenu xvisFileMenu, xvisObjectMenu, xvisWindowMenu;
	extern xvPulldownMenu xvisGenerateMenu, xvisHelpMenu;
	extern xvButton xvisNew, xvisOpen, xvisSave, xvisCC, xvisInfo, xvisEditMenu, xvisNewForm;
	extern xvLabel xvisCount;
	extern char countvalue[];
	extern char project_name[], project_path[];
	extern int project_list[];

extern xvWindow xvisTools;
	extern xvLabel toollabel;
	extern xvToggle xTimer, xBitmap, xDrop, xView, xToggle;
	extern xvToggle xVScroll, xHScroll, xPanner, xList;
	extern xvToggle xLabel, xEdit, xButton, xRepeater, xBox;
	extern xvToggle xFileList, xDirList, xForm, xPointer;
	
extern xvWindow xvisProps;
	extern xvView propview;
	extern xvForm propform;
	extern xvList proplist, valuelist;
	extern xvLabel proplabel, propobject;
	extern xvButton propchange;
	extern xvEditField propedit;
	extern objectType prop_object_type;
	extern int prop_object_index;
	extern int selected_index;
	extern char propeditbuffer[];
	extern char objname[];

extern xvWindow xvisEvents;
	extern xvToggle eventtoggles[];

extern xvPopupShell xvisColours;
	extern xvForm colourform;
	extern xvView colourview;
	extern xvList colourlist;
	extern xvLabel colourexample;
	extern xvButton colourok, colourcancel, colourdefault;
	extern int colourmode;

extern xvWindow xvisMake;
	extern xvEditField maketext;
	extern xvButton makequit;
	extern char *makebuffer;

extern xvPopupShell xvisCursors;
	extern xvList xvisCursorList;

extern xvWindow xvisGrid;
	extern xvLabel xlab, ylab;
	extern xvButton labok, labcancel;
	extern xvEditField xval, yval;

extern xvPopupShell xvisPath;
	extern xvForm xvisPathForm;
	extern xvLabel xvisPathLabel;
	extern xvFileList pathlist;
	extern xvMsgBox pathAlert;
	extern xvButton pathok, pathcancel;
	extern char provpath[];
	extern char cwd[];

extern xvPopupShell xvisFonts;
	extern xvFontList masterfontlist;
	extern xvView fontview;
	extern xvList fontlist;
	extern xvButton fontok, fontcancel;
	extern xvLabel fontexample;
	extern xvForm fontform;
	extern xvFont fontfont;

extern xvWindow xvisObjList;
	extern xvForm objform;
	extern xvView objview;
	extern xvList objlist;
	extern xvButton objok, objcancel;
	extern int num_objects;
	extern int selected_ref;
	extern struct object_listdata objectinfo[];
	extern char *objectlist[];

extern xvWindow xvisProjList;
	extern xvView projview;
	extern xvList projlist;
	extern xvButton projOpen, projDelete;
	extern int num_proj;
	extern char *projectlist[];

extern xvPopupShell xvisCallList;
	extern xvForm callform;
	extern xvView callview;
	extern xvList calllist;
	extern xvButton callclose;
	extern int num_calls;
	extern char *calls_list[];
	
extern xvWindow xvisMenus;
	extern xvEditField menuCaption, menuName, menuCallback;
	extern xvLabel menuCaptionLabel, menuNameLabel, menuCallbackLabel;
	extern xvView menuView;
	extern xvList menuList;
	extern xvButton menuOK, menuCancel;
	extern xvButton menuAdd, menuAddSep, menuUpdate, menuRemove, menuClear;
	extern xvButton menuDown, menuUp, menuMain, menuSub;
	extern xvButton menuFont;
	extern xvButton menuBack, menuFore;
	extern xvButton menuEditCallback;

extern xvWindow xvisGenerate;
	extern xvButton genOK, genCancel;
	extern xvLabel AppNameLabel, MakeChoiceLabel, MakeFileLabel, LinkChoiceLabel, LibLabel;
	extern xvToggle MakeYes, MakeNo;
	extern xvToggle MakeStatic, MakeShared;
	extern xvEditField AppName, MakeFile, Libs;

extern xvEditWindow xvisError;
	extern char errorbuffer[];

extern xvPopupShell xvisAbout;
	extern xvForm aboutform;
	extern xvLabel aboutmap;
	extern xvLabel aboutlabel;
	extern xvButton aboutbutton;

extern xvPopupMenu objectpopup;
extern xvCore *CurrentClass;
extern xvSizer *sizer;
extern Boolean objectpopupopen;

// ---------------------------------------------
// Global Vars & data

extern Boolean xvisExit, ToolsVisible, PropsVisible, MenusVisible, ColoursVisible;
extern Boolean EventsVisible, ProjectListVisible, ErrorsVisible;
extern Boolean establishedFileFlag;

extern objectType CurrentObjectType, CurrentParentType;
extern int CurrentObject, CurrentParent;
extern int ObjectBeingSized, ObjectBeingSizedParent;
extern xvWindow *CurrentObjectWindow;
extern char CurrentObjectName[];
extern objectType ObjectBeingSizedParentType;

extern objectType CurrentToolType;
extern char *CurrentToolName;

extern int CurrentX, CurrentY;
extern int RelativeX, RelativeY;

extern int gridX, gridY;
extern Boolean snapToGrid, showGrid;

// ---------------------------------------------
// Globals for Resources

#define xvNnone			"none"
#define xvNobjectName		"objectName"
#define xvNtimerInterval	"timerInterval"
#define xvNwindowName		"windowName"
#define xvNwindowIcon		"windowIcon"
#define xvNpath			"path"
#define xvNbitmapFile		"bitmapFile"
#define xvNleftBitmap		"leftBitmap"
#define xvNobjectRef		"objectRef"
#define xvNfixed			"fixed"
#define xvNiconic			"iconic"
#define xvNicon			"icon"
#define xvNopenAtStart		"openAtStart"
#define xvNenabled			"enabled"
#define xvNvisible			"visible"
#define xvNautowrap		"autowrap"
#define xvNwrapmode		"wrapmode"
#define xvNVscrollmode		"Vscrollmode"
#define xvNHscrollmode		"Hscrollmode"
#define xvNresizemode		"resizemode"
#define xvNecho			"echo"
#define xvNctrl			"ctrl"
#define xvNeditmode		"editmode"
#define xvNshapeStype		"shapeStyle"

extern callType* object_call_types[];
extern char * *object_call_names[];
extern int object_call_number[];
extern char* *object_resources[];
extern char* *object_resource_types[];
extern resourceType* object_data_types[];
extern int object_resource_number[];
extern struct resource_store resource_values[];
extern int resource_full;
extern char* *resvallist;
extern Boolean resvalflag;

// ---------------------------------------------
// File Mode defines

#define xvPROJECTLOAD	0
#define xvPROJECTSAVE	1
#define xvFORMLOAD		2
#define xvFORMSAVE		3
#define xvFORMSAVEAS	4

// ---------------------------------------------
// Extern Callbacks

extern void func_dummy(Widget aw, XtPointer client, XtPointer call);
extern void func_exit(Widget aw, XtPointer client, XtPointer call);
extern void func_timer(XtPointer cdata, XtIntervalId *id);

// ---------------------------------------------
// Extern functions in other modules which require global definition

extern void buildControlWindow();
extern void buildToolsWindow();
extern void buildCursorWindow();
extern void buildColourWindow(XtCallbackProc confirm);
extern void buildFontsWindow(XtCallbackProc confirm);
extern void buildPathWindow(XtCallbackProc ok_call);
extern void buildSelectFileWindow(String name, XtCallbackProc okcallback, String mask, Boolean activemask=FALSE, Boolean allowDirchange=TRUE);
extern void buildObjectListWindow(XtCallbackProc execproc);
extern void buildEventsWindow();
extern void buildMenuWindow();
extern void buildCallsWindow(int mode);
extern void openMenuWindow();
extern void buildProjectListWindow();
extern void buildCodeGenWindow();
extern void updateProjectListWindow();
extern void updateCallbackCode(String oldname, String newname);
extern void buildAboutWindow();
extern void buildGridWindow();
extern void openGridWindow();

extern void SetupPropsWindow(objectType xtype, char **resource_vals, Boolean build=FALSE, int obj=-1);
extern void SetupPropertiesList(int obj, Boolean newlist);
extern void openObjectListWindow(objectType listobject, int current);

extern int CreateObject(objectType object);
extern void DeleteNamedObject(String name);
extern void removeSizer();

extern objectType ToolNameToType(char *token);
extern int ObjectByName(String name);
extern int firstAvailableSlot();
extern void SetCounterValue();
extern int findFirstProjectEntry(int which=-1);
extern Pixel stringToPixel(xvCore & object, String colname);
extern int findFirstEditEntry();
extern int findFirstModuleEntry();
extern Boolean UnsavedData();
extern void drawGrid(int obj);

extern void commenceCodeGeneration(Boolean codeonly);
extern void buildProgram(Boolean buildall);
extern void runProgram();

extern void makeNewModule();
extern void loadModule();
extern void saveModule(Boolean saveas);

extern void setEventsData(unsigned long bits);
extern void controlEventsData(Boolean sense);
extern void clearEventsData();
extern void constructVisualMenu(int target, int numberof);

extern void recoverResources(int numres, int thisobj);
extern void setNewResourceValue(Boolean visual=TRUE);
extern void setResourceExec(Boolean visual=TRUE);
extern void execFileAction(int fileAction, String name);

extern void WindowEvent(Widget aw, XtPointer client, register XConfigureEvent *event, Boolean *cont);
extern void enter_call(Widget aw, XtPointer client, register XEnterWindowEvent *event, Boolean *cont);
extern void leave_call(Widget aw, XtPointer client, register XLeaveWindowEvent *event, Boolean *cont);
extern void ObjectButtonEvent(Widget aw, XtPointer client, register XButtonEvent *event, Boolean *cont);

#endif // _Vxvis_h
 