
/*     *******************************************************************************
	XVisual GUI Development System for Linux/X-Window
	Copyright (C) 1996 Steve Carrie.

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
        ******************************************************************************* */

/* =============================================
	xvis user interface builder for Athena widgets using
	xv class library.

	xvisProps segment

	V1.0BETA  - last edited 01/07/96
    ============================================= */

// ---------------------------------------------
// Local Include file declarations

#include "xvis.h"
#include <string.h>
#include <stdio.h>
#include <ctype.h>

// ---------------------------------------------
// Local Image files


// ---------------------------------------------
// Local Forward Function declarations

void buildResourceValuesList(objectType obj, int thisobj);
void recoverResources(int numres, int thisobj);
Boolean objectExists(String newname);
void func_changeval(Widget aw, XtPointer client, XtPointer call);
void func_editcall(Widget aw, XtPointer client, XtPointer call);

// ---------------------------------------------
// Local Variable & Data declarations

Boolean replaceflag=FALSE;
int maptype=0;
extern Boolean dirChanged;

// ---------------------------------------------
// Local Callbacks

// ******
// alert messagebox callback
//
void alert_call(Widget aw, XtPointer client, XtPointer call)
{
	alert.Close();
}

// ---------------------
// Changes toolbox for proplist selection
//
void alterToolbox(XawListReturnStruct *prop)
{
	char *tempsp;
	char * *rnames=object_resources[prop_object_type];

	proplist.Highlight(selected_index);
	valuelist.Highlight(selected_index);

	proplabel.SetLabel(rnames[selected_index]);

	switch(resource_values[selected_index].r_type) {
		case XVLocBoolean:
		case XVBoolean:
 		case XVPixel:
		case XVCursor:
		case XVFont:
		case XVBitmapFile:
		case XVObject:
		case XVPath:
		case XVResize:
		case XVhScroll:
		case XVvScroll:
		case XVEditmode:
		case XVShapeStyle:
		case XVJustify:
		case XVWrap:
			propedit.SetSensitive(FALSE);	
			propedit.SetBackground("gray50");
			propchange.SetSensitive(TRUE);
			propchange.SetBackground("yellow");
			tempsp=NULL;
			propedit.SetString(tempsp);
			break;
		case XVVoid:
			propedit.SetSensitive(FALSE);
			propedit.SetBackground("gray50");
			propchange.SetSensitive(FALSE);
			propchange.SetBackground("gray50");
			tempsp=NULL;
			propedit.SetString(tempsp);
			break;
		default:
			propedit.SetSensitive(TRUE);
			propedit.SetBackground("yellow");
			propchange.SetSensitive(FALSE);
			propchange.SetBackground("gray50");
			strcpy(propeditbuffer, resource_values[selected_index].r_value);
			propedit.SetString(propeditbuffer);
			break;
	}
}

void func_editcall(Widget aw, XtPointer client, XtPointer call)
{
	XawTextPosition tstart, tend;
	char *c;

	if (replaceflag) {
		propedit.Position(&tstart, &tend);
		if (tstart < tend) {
			propeditbuffer[0]='\0';
			propedit.SetString(propeditbuffer);		
		}
		replaceflag=FALSE;
	}
	
}

// ******
// proplist reacts to the selection of properties in the list and displays appropriate
// editing tool
//
void func_proplist(Widget aw, XtPointer client, XtPointer call)
{

	XawListReturnStruct *prop=(XawListReturnStruct *) call;
	selected_index=prop->list_index;
	alterToolbox(prop);	
}


// *********
// Valuelist reacts to click on value list.
//
void func_valuelist(Widget aw, XtPointer client, XtPointer call)
{
	XawListReturnStruct *prop=(XawListReturnStruct *) call;
	selected_index=prop->list_index;
	alterToolbox(prop);
	func_changeval(aw, client, call);
}

// **************
// func_objok is for object resourcechanges
//

void func_objok(Widget aw, XtPointer client, XtPointer call)
{
	if (prop_object_index==selected_ref) strcpy(propeditbuffer, "(None)");
	xvisObjList.RemoveGrab();
	xvisObjList.Close();
	setNewResourceValue();
}

void func_fileok(Widget aw, XtPointer client, XtPointer call)
{
	strcpy(provpath, xvisSelFile.SelectedFile(propeditbuffer));
	if (! dirChanged) strcpy(propeditbuffer, provpath);
	xvisSelectFile.RemoveGrab();
	xvisSelectFile.Close();
	setNewResourceValue();
}

// ****************
// confirm font call

void func_fontok_call(Widget aw, XtPointer client, XtPointer call)
{
	xvisFonts.Close();
	setNewResourceValue();
}

void func_propcolour(Widget aw, XtPointer client, XtPointer call)
{
	xvisColours.Close();
	setNewResourceValue();
}

void func_pathok(Widget aw, XtPointer client, XtPointer call)
{
	strcpy(propeditbuffer, provpath);
	xvisPath.Close();
	setNewResourceValue();
}

void bitmap_alert_yes(Widget aw, XtPointer client, XtPointer call)
{
	alert.Close();
	strcpy(propeditbuffer, "(None)");
	setNewResourceValue();
	
}

void bitmap_alert_no(Widget aw, XtPointer client, XtPointer call)
{
	alert.Close();
	buildSelectFileWindow("Select Bitmap File", func_fileok, "");
	xvisSelFile.NewPath(cwd);
	xvisSelectFile.Open();
	xvisSelectFile.AddGrab(FALSE, FALSE);
}

void cursor_alert_yes(Widget aw, XtPointer client, XtPointer call)
{
	alert.Close();
	strcpy(propeditbuffer, "(None)");
	setNewResourceValue();
	objects[prop_object_index].pcore->SetCursorName(NULL);
}

void cursor_alert_no(Widget aw, XtPointer client, XtPointer call)
{
	alert.Close();
	objects[prop_object_index].pcore->SetCursorName(NULL);
	xvisCursors.Open();
}

void func_changeval(Widget aw, XtPointer client, XtPointer call)
{
	int tempval;
	XawTextPosition vpos;
	char *s;
	char * *rnames=object_resources[prop_object_type];

	if ((objects[prop_object_index].object_type == XVForm) && 
		(objects[prop_object_index].num_children == 0)) {
		alert.init("Can't Set Properties", xvisProps, 
			"Can't set properties of an empty form", XVMSGSIMPLEFATAL, "SkyBlue",
			alert_call, alert_call);
		alert.Open((int)screenWidth/2, (int)screenHeight/2);
		return;
	}

	switch(resource_values[selected_index].r_type) {
		case XVLocBoolean:
		case XVBoolean:
			if (strcmp(resource_values[selected_index].r_value, "TRUE")==0)
				strcpy(propeditbuffer, "FALSE");
			else
				strcpy(propeditbuffer, "TRUE");
			setNewResourceValue();
			break;
		case XVPixel:
			if (strcmp(rnames[selected_index], "Background")==0) 
				colourmode=BCOLOUR;
			if (strcmp(rnames[selected_index], "Foreground")==0)
				colourmode=FCOLOUR;
			if (strcmp(rnames[selected_index], "Top Shadow Colour")==0) 
				colourmode=TSCOLOUR;
			if (strcmp(rnames[selected_index], "Bottom Shadow Colour")==0)
				colourmode=BSCOLOUR;

			buildColourWindow(func_propcolour);
			xvisColours.Open();
			break;
		case XVCursor:
			if (strcmp(objects[prop_object_index].obj_cursor, "(None)") != 0) {
				alert.init("Cursor property already assigned.", xvisControl, 
					"A Cursor property has already been assigned to this resource.\nDo you want to clear it?\n", XVMSGSIMPLECHOICE, 
					"SkyBlue", cursor_alert_yes, cursor_alert_no);
				alert.Open();
			}
			else
				xvisCursors.Open();
			break;
		case XVFont:
			buildFontsWindow(func_fontok_call);
			xvisFonts.Open();
			break;
		case XVPath:
			buildPathWindow(func_pathok);
			pathlist.SetNewPath(resource_values[selected_index].r_value, XVDIRLIST, TRUE);
			strcpy(provpath, resource_values[selected_index].r_value);
			xvisPath.Open();
			break;
		case XVBitmapFile:
			tempval=0;
			maptype=0;
			if (strcmp(rnames[selected_index], "Bitmap File")==0) {
				tempval=strcmp(objects[prop_object_index].obj_bitref, "(None)");
				maptype=1;
			}
			else
				if (strcmp(rnames[selected_index], "Left Bitmap")==0) {
					tempval=strcmp(objects[prop_object_index].obj_leftbitref, "(None)");
					maptype=2;
				}
				else
					if (strcmp(rnames[selected_index], "Icon")==0) {
						tempval=strcmp(objects[prop_object_index].obj_icon, "(None)");
						maptype=3;
					}

			if (tempval!=0) {
				alert.init("Bitmap object already assigned.", xvisControl, 
					"A bitmap object has already been assigned to this resource.\nDo you want to clear it?", XVMSGSIMPLECHOICE, 
					"SkyBlue", bitmap_alert_yes, bitmap_alert_no);
				alert.Open();
			}
			else {
				buildSelectFileWindow("Select Bitmap File", func_fileok, "");
				xvisSelFile.NewPath(cwd);
				xvisSelectFile.Open();
				xvisSelectFile.AddGrab(FALSE, FALSE);
			}
			break;
		case XVObject:
			buildObjectListWindow(func_objok);
			openObjectListWindow(objects[prop_object_index].object_type, -1);	
			break;
		case XVResize:
			tempval=objects[prop_object_index].resize_mode;			
			tempval++;
			if (tempval>=4) tempval=0;
			objects[prop_object_index].resize_mode=tempval;			
			setNewResourceValue();
			break;
		case XVWrap:
			tempval=objects[prop_object_index].wrap_mode;			
			tempval++;
			if (tempval>=3) tempval=0;
			objects[prop_object_index].wrap_mode=tempval;			
			setNewResourceValue();
			break;
		case XVhScroll:
			tempval=objects[prop_object_index].hscroll_mode;			
			tempval++;
			if (tempval>=3) tempval=0;
			objects[prop_object_index].hscroll_mode=tempval;			
			setNewResourceValue();
			break;
		case XVvScroll:
			tempval=objects[prop_object_index].vscroll_mode;			
			tempval++;
			if (tempval>=3) tempval=0;
			objects[prop_object_index].vscroll_mode=tempval;			
			setNewResourceValue();
			break;
		case XVEditmode:
			tempval=objects[prop_object_index].edit_mode;			
			tempval++;
			if (tempval>=3) tempval=0;
			objects[prop_object_index].edit_mode=tempval;			
			setNewResourceValue();
			break;
		case XVShapeStyle:
			tempval=objects[prop_object_index].shape;			
			tempval++;
			if (tempval>=4) tempval=0;
			objects[prop_object_index].shape=tempval;			
			setNewResourceValue();
			break;
		case XVJustify:
			tempval=objects[prop_object_index].justify;			
			tempval++;
			if (tempval>=3) tempval=0;
			objects[prop_object_index].justify=tempval;			
			setNewResourceValue();
			break;
		default:
			xvisProps.MovePointer(100,50);
			s=propedit.ReturnString(s);		
			vpos=(XawTextPosition) strlen(s);
			propedit.SetSelection(0L, vpos);	
			propedit.InsertionPoint(vpos);
			replaceflag=TRUE;
			break;
	}
}

// ---------------------------------------------
// Local Events

//************
// Dataentry is action proc for editfield - gets value from the field when
// RETURN is pressed
//
void Dataentry(Widget w, XEvent *event, String *params, Cardinal *num)
{
	XawTextPosition start, end;
	XawTextBlock block;
	XawTextPosition vpos;
	char *s;

	if ((objects[prop_object_index].object_type == XVForm) && 
		(objects[prop_object_index].num_children == 0)) {
		alert.init("Can't Set Properties", xvisProps, 
			"Can't set properties of an empty form", XVMSGSIMPLEFATAL, "SkyBlue",
			alert_call, alert_call);
		alert.Open((int)screenWidth/2, (int)screenHeight/2);
		return;
	}

	propedit.Position(&start, &end);
	propedit.SourceRead(0, &block, 40);
	strncpy(propeditbuffer, block.ptr, end);
	propeditbuffer[end]='\0';
	propedit.UnsetSelection();
	while (isspace(propeditbuffer[0])) 
		for (int i=1; i<strlen(propeditbuffer)+1; i++)
			propeditbuffer[i-1]=propeditbuffer[i];

	if (strlen(propeditbuffer) == 0) {
		strcpy(propeditbuffer, resource_values[selected_index].r_value);
		propedit.SetString(propeditbuffer);
		setNewResourceValue();
		s=propedit.ReturnString(s);		
		vpos=(XawTextPosition) strlen(s);
		propedit.SetSelection(0L, vpos);	
		propedit.InsertionPoint(vpos);
		replaceflag=TRUE;
	}
	else
		setNewResourceValue();

}

// ---------------------------------------------
// Local Functions

// ***********
// buildPropsWindow is initial window contruction routine
//
void buildPropsWindow(int nres, char **resources, char **resource_vals, char **resource_types)
{
	xvisProps.init("Properties", "xvisProps", xvisContext);
	xvisProps.SetBackground("gray70");
	xvisProps.SetResizable(TRUE);
	xvisProps.SetWindowResource(XtNgeometry, (XtArgVal)"+700+250");
	xvisProps.SetWindowResource(XtNmaxWidth, 255);
	xvisProps.SetWindowResource(XtNminWidth, 255);
 	xvisProps.SetWindowResource(XtNiconPixmap, 
		xvisProps.SetPixmapInfo(xvis_xpm, NULL));

	propobject.init("No Object Selected", xvisProps);
	propobject.SetDistance(3, 1);
	propobject.SetBackground("SkyBlue");
	propobject.SetFixedPosition(TRUE);
	
	proplabel.init( " ", xvisProps);
	proplabel.SetDistance(3, 1);
	proplabel.SetRelVertical(propobject);
	proplabel.SetBackground("SkyBlue");
	proplabel.SetFixedPosition(TRUE);
	
	propchange.init("Change...", xvisProps, func_changeval);
	propchange.SetRelative(proplabel, propobject);
	propchange.SetDistance(1, 1);
	propchange.SetBackground("yellow");
	propchange.SetShadowWidth(0);
	propchange.SetBorderWidth(0);
	propchange.SetFixedPosition(TRUE);
	propchange.SetHeight(proplabel.GetHeight());
	propchange.SetBeNiceToColormap(FALSE);
	int sy=propchange.GetWidth();
	int sx=propchange.GetHeight();

	propedit.init("propedit", xvisProps, XawtextEdit, sx, sy, XawAsciiString, 
		propeditbuffer, FALSE,  func_editcall, NULL);
	propedit.SetRelVertical(proplabel);
	propedit.SetDistance(3,1);
	propedit.SetBackground("yellow");
	propedit.SetFixedPosition(TRUE);
	propedit.SetResource(XtNscrollHorizontal, XawtextScrollNever);
	propedit.SetResource(XtNscrollVertical, XawtextScrollNever);
	propeditbuffer[0]='\0';
	propedit.SetString(propeditbuffer);

	xvisContext.AddAction("dataentry", (XtActionProc) Dataentry);

	XtTranslations trans=XtParseTranslationTable(
		"<Key>Return: select-all()  dataentry() \n\
		!Meta<Key> i: no-op()");
	propedit.OverrideTranslations(trans);

	propview.init("propview", xvisProps, TRUE, FALSE, TRUE);
	propview.SetRelVertical(propedit);
	propview.SetDistance(1, 1);
	propview.SetResizable(TRUE);
	propview.SetFixedPosition(TRUE);
	
	propform.init("propform", propview, TRUE, TRUE);
	propform.SetDistance(0, 0);
	propform.SetDefaultDistance(0);
	propform.SetResizable(TRUE);
	propform.SetFixedPosition(TRUE);

	proplist.init("proplist", propform, func_proplist, nres, resources, 1);
	proplist.SetDistance(1, 1);
	proplist.SetWidth(proplist.GetLongest()+20);
	proplist.SetResizable(TRUE);
	proplist.SetFixedPosition(TRUE);

	valuelist.init("propvalues", propform, func_valuelist, nres, resource_vals, 1);
	valuelist.SetDistance(0, 1);
	valuelist.SetRelHorizontal(proplist);
	valuelist.SetWidth(valuelist.GetLongest()+20);
	valuelist.SetResizable(TRUE);
	valuelist.SetFixedPosition(TRUE);

	proplabel.SetWidth(proplist.GetWidth());
	propobject.SetWidth(propform.GetWidth());
	propedit.SetWidth(propform.GetWidth());
	propchange.SetWidth(valuelist.GetWidth());
	
	PropsVisible=TRUE;
}

// ------------------------------------

// ***********
// changePropsWindow is called to change the contents of the list
//
void changePropsWindow(int nres, char **resources, char **resource_vals, char **resource_types)
{
	Dimension wid1=proplist.GetWidth();
	Dimension wid2=valuelist.GetWidth();
	Dimension hgt=propview.GetHeight();
	Dimension winhgt;

	XtUnmanageChild(xvisProps.ID());
	propform.ManageChildren(FALSE);
		
	proplist.SetList(nres, resources);
	proplist.SetWidth(wid1);

	valuelist.SetList(nres, resource_vals);
	valuelist.SetWidth(wid2);

	propview.SetHeight(proplist.GetHeight()+3);
	int hgtdiff=propview.GetHeight()-hgt;

	xvisProps.SetHeight(xvisProps.GetHeight()+hgtdiff);
	XtVaGetValues(xvisProps.GetParent(), XtNheight, &winhgt, NULL);
	XtVaSetValues(xvisProps.GetParent(), XtNheight, winhgt+hgtdiff ,NULL);
	xvisProps.SetWindowResource(XtNmaxHeight, winhgt+hgtdiff);
	xvisProps.SetWindowResource(XtNminHeight, winhgt+hgtdiff);

	propform.ManageChildren(TRUE);
	XtManageChild(xvisProps.ID());
}

// ------------------------------------

// ************
// SetupPropsWindow is the HIGHLEVEL call to build the properties window
// and will call buildPropsWindow or changePropsWindow
//
void SetupPropsWindow(objectType xtype, char **resource_vals, Boolean build, int obj)
{
	if (obj!=-1) { 
		sprintf(objname, "Object '%s'. Type %s", objects[obj].name, 
			objectName[objects[obj].object_type]);
		propobject.SetLabel(objname);
	}
	else 
		if (!build) {
			sprintf(objname, "No Object Selected");
			propobject.SetLabel(objname);
		}

	if (build) 
		buildPropsWindow(object_resource_number[xtype], 
			object_resources[xtype], resource_vals, object_resource_types[xtype]);
	else {	
		changePropsWindow(object_resource_number[xtype], 
			object_resources[xtype], resource_vals, object_resource_types[xtype]);
	}
}

// ------------------------------------

// ****************
// SetupPropertiesList is used to change the properties list to a new object (obj). If this
// if a complete change, newlist=TRUE.
//
void SetupPropertiesList(int obj, Boolean newlist)
{
	if (obj==-1) {
		prop_object_type=XVNone;
		propedit.SetSensitive(FALSE);
		propedit.SetBackground("gray50");
		propchange.SetSensitive(FALSE);
		propchange.SetBackground("gray50");
	}
	else
		prop_object_type=objects[obj].object_type;

	prop_object_index=obj;
	
	if (newlist) {
		if (resvalflag) {
			delete resvallist;
			resvalflag=FALSE;
		}

		resvallist=new char*[object_resource_number[prop_object_type]];
		resvalflag=TRUE;
	}

	buildResourceValuesList(prop_object_type, obj);

	SetupPropsWindow(prop_object_type, resvallist, FALSE, obj);	

	setEventsData(objects[obj].events);
	
	if (newlist) {
		propeditbuffer[0]='\0';
		propedit.SetResource("string", (XtArgVal) propeditbuffer);
		proplist.Highlight(0);		
		selected_index=0;
		if (obj!=-1) {
			propedit.SetSensitive(TRUE);	
			propedit.SetBackground("yellow");
			propchange.SetSensitive(FALSE);
			propchange.SetBackground("gray50");
			strcpy(propeditbuffer, resource_values[0].r_value);
			propedit.SetResource("string", (XtArgVal) propeditbuffer);
		}
	}
}

// -----------------------------------------

// ******************
// buildResourceValuesList constructs the resource structure and reads the resource
// values from object 'thisobj' of type 'obj'
//
void buildResourceValuesList(objectType obj, int thisobj)
{
	int numres=object_resource_number[obj];
	char* *types=object_resource_types[obj];
	resourceType *datatypes=object_data_types[obj];

	for (int i=0; i<numres; i++) {
		resource_values[i].r_restype=types[i];
		resource_values[i].r_type=datatypes[i];
	}
	recoverResources(numres, thisobj);		

	for (int i=0; i<numres; i++)
		resvallist[i]=resource_values[i].r_value;
}

// -----------------------------------------

// ****************
// recoverResources reads 'numres' resources from the object 'thisobj'
//
void recoverResources(int numres, int thisobj)
{
	int otype=objects[thisobj].object_type;
	int ref;

	for (int i=0; i<numres; i++) {
		switch(resource_values[i].r_type) {
			case XVPixel:
				if (strcmp(resource_values[i].r_restype, XtNbackground)==0) {
					sprintf(resource_values[i].r_value, "%s", 
						objects[thisobj].background_colour);
				}
				if (strcmp(resource_values[i].r_restype, XtNforeground)==0) {
					sprintf(resource_values[i].r_value, "%s", 
						objects[thisobj].foreground_colour);
				}
				if (strcmp(resource_values[i].r_restype, XtNborderColor)==0) {
					sprintf(resource_values[i].r_value, "%s", 
						objects[thisobj].border_colour);
				}
				if (strcmp(resource_values[i].r_restype, XtNtopShadowPixel)==0) {
					sprintf(resource_values[i].r_value, "%s", 
						objects[thisobj].topshadow_colour);
				}
				if (strcmp(resource_values[i].r_restype, XtNbottomShadowPixel)==0) {
					sprintf(resource_values[i].r_value, "%s", 
						objects[thisobj].bottomshadow_colour);
				}
				break;
			case XVDimension:
				switch(otype) {
					case XVList:
						objects[thisobj].plist->xvView::GetResource(
							resource_values[i].r_restype, 
							(XtArgVal *) &resource_values[i].r_dimension);
						break;
					case XVFileList:
						objects[thisobj].pfilelist->xvView::GetResource(
							resource_values[i].r_restype, 
							(XtArgVal *) &resource_values[i].r_dimension);
						break;
					case XVDirList:
						objects[thisobj].pdirlist->xvView::GetResource(
							resource_values[i].r_restype, 
							(XtArgVal *) &resource_values[i].r_dimension);
						break;
					default:
						objects[thisobj].pcore->GetResource(
							resource_values[i].r_restype, 
							(XtArgVal *) &resource_values[i].r_dimension);
						break;
				}
				sprintf(resource_values[i].r_value, "%d", resource_values[i].r_dimension);
				break;
			case XVPosition:
				switch(otype) {
					case XVList:
						objects[thisobj].plist->xvView::GetResource(
							resource_values[i].r_restype, 
							(XtArgVal *) &resource_values[i].r_position);
						break;
					case XVFileList:
						objects[thisobj].pfilelist->xvView::GetResource(
							resource_values[i].r_restype, 
							(XtArgVal *) &resource_values[i].r_position);
						break;
					case XVDirList:
						objects[thisobj].pdirlist->xvView::GetResource(
							resource_values[i].r_restype, 
							(XtArgVal *) &resource_values[i].r_position);
						break;
					default:
						objects[thisobj].pcore->GetResource(
							resource_values[i].r_restype, 
							(XtArgVal *) &resource_values[i].r_position);
						break;
				}
				sprintf(resource_values[i].r_value, "%d", resource_values[i].r_position);
				break;
			case XVInt:
				objects[thisobj].pcore->GetResource(resource_values[i].r_restype, 
					(XtArgVal *) &resource_values[i].r_int);
				sprintf(resource_values[i].r_value, "%d", resource_values[i].r_int);
				break;
			case XVString:
				objects[thisobj].pcore->GetResource(resource_values[i].r_restype, 
					(XtArgVal *) &resource_values[i].r_string);
				sprintf(resource_values[i].r_value, "%s", resource_values[i].r_string);
				break;
			case XVCursor:
				strcpy(resource_values[i].r_value, objects[thisobj].obj_cursor);
				break;
			case XVBoolean:
				objects[thisobj].pcore->GetResource(resource_values[i].r_restype, 
					(XtArgVal *) &resource_values[i].r_boolean);
				if (resource_values[i].r_boolean == TRUE)					
					strcpy(resource_values[i].r_value, "TRUE");
				else
					strcpy(resource_values[i].r_value, "FALSE");
				break;
			case XVLocBoolean:
				if (strcmp(resource_values[i].r_restype, xvNfixed)==0) {
					if (objects[thisobj].fixed_size == TRUE)					
						strcpy(resource_values[i].r_value, "TRUE");
					else
						strcpy(resource_values[i].r_value, "FALSE");
				}
				if (strcmp(resource_values[i].r_restype, xvNiconic)==0) {
					if (objects[thisobj].iconic == TRUE)					
						strcpy(resource_values[i].r_value, "TRUE");
					else
						strcpy(resource_values[i].r_value, "FALSE");
				}
				if (strcmp(resource_values[i].r_restype, xvNopenAtStart)==0) {
					if (objects[thisobj].open_at_startup == TRUE)					
						strcpy(resource_values[i].r_value, "TRUE");
					else
						strcpy(resource_values[i].r_value, "FALSE");
				}
				if (strcmp(resource_values[i].r_restype, xvNenabled)==0) {
					if (objects[thisobj].enabled == TRUE)					
						strcpy(resource_values[i].r_value, "TRUE");
					else
						strcpy(resource_values[i].r_value, "FALSE");
				}
				if (strcmp(resource_values[i].r_restype, xvNvisible)==0) {
					if (objects[thisobj].visible == TRUE)					
						strcpy(resource_values[i].r_value, "TRUE");
					else
						strcpy(resource_values[i].r_value, "FALSE");
				}
				if (strcmp(resource_values[i].r_restype, xvNautowrap)==0) {
					if (objects[thisobj].autowrap == TRUE)					
						strcpy(resource_values[i].r_value, "TRUE");
					else
						strcpy(resource_values[i].r_value, "FALSE");
				}
				if (strcmp(resource_values[i].r_restype, xvNecho)==0) {
					if (objects[thisobj].echo == TRUE)					
						strcpy(resource_values[i].r_value, "TRUE");
					else
						strcpy(resource_values[i].r_value, "FALSE");
				}
				if (strcmp(resource_values[i].r_restype, xvNctrl)==0) {
					if (objects[thisobj].ctrl == TRUE)					
						strcpy(resource_values[i].r_value, "TRUE");
					else
						strcpy(resource_values[i].r_value, "FALSE");
				}
				break;
			case XVName:
				strcpy(resource_values[i].r_value, objects[thisobj].name);
				break;
			case XVWinName:
				objects[thisobj].pwindow->GetWindowResource(resource_values[i].r_restype,
					(XtArgVal *) &resource_values[i].r_string);
				sprintf(resource_values[i].r_value, "%s", resource_values[i].r_string);
				break;
			case XVWinSize:
				objects[thisobj].pwindow->GetWindowResource(resource_values[i].r_restype,
 					(XtArgVal *) &resource_values[i].r_dimension);
				sprintf(resource_values[i].r_value, "%d", resource_values[i].r_dimension);
				if (strcmp(resource_values[i].r_restype, XtNwidth)==0) 
					objects[thisobj].winwidth=resource_values[i].r_dimension;
				if (strcmp(resource_values[i].r_restype, XtNheight)==0)
					objects[thisobj].winheight=resource_values[i].r_dimension;
				break;
			case XVWinPositionX:
			case XVWinPositionY:
				objects[thisobj].pwindow->GetWindowResource(resource_values[i].r_restype, 
					(XtArgVal *) &resource_values[i].r_position);
				sprintf(resource_values[i].r_value, "%d", resource_values[i].r_position);
				if (resource_values[selected_index].r_type == XVWinPositionX)
					objects[thisobj].winxpos=resource_values[i].r_position;
				else
					objects[thisobj].winypos=resource_values[i].r_position;
				break;
			case XVFont:
				strcpy(resource_values[i].r_value, objects[thisobj].obj_font);
				break;
			case XVVoid:
				strcpy(resource_values[i].r_value, "-Void-");
				break;
			case XVPath:
				strcpy(resource_values[i].r_value, objects[thisobj].obj_path);
				break;
			case XVBitmapFile:
				if (strcmp(resource_values[i].r_restype, XtNleftBitmap)==0) 
					strcpy(resource_values[i].r_value, objects[thisobj].leftbitmap);
				else 
					if (otype==XVWindow)
						strcpy(resource_values[i].r_value, objects[thisobj].obj_icon);
					else
						strcpy(resource_values[i].r_value, objects[thisobj].obj_path);
				break;
			case XVObject:
				ref=objects[thisobj].object_ref3;
				if (ref==-1)
					strcpy(resource_values[i].r_value, "(None)");
				else
					strcpy(resource_values[i].r_value, objects[ref].name);	
				break;
			case XVTime:
				sprintf(resource_values[i].r_value, "%d", objects[thisobj].object_time);				
				break;
			case XVResize:
				strcpy(resource_values[i].r_value, resizeinfo[objects[thisobj].resize_mode]);
				break;
			case XVvScroll:
				strcpy(resource_values[i].r_value, scrollinfo[objects[thisobj].vscroll_mode]);
				break;
			case XVhScroll:
				strcpy(resource_values[i].r_value, scrollinfo[objects[thisobj].hscroll_mode]);
				break;
			case XVWrap:
				strcpy(resource_values[i].r_value, wrapinfo[objects[thisobj].wrap_mode]);
				break;
			case XVEditmode:
				strcpy(resource_values[i].r_value, editmodeinfo[objects[thisobj].edit_mode]);
				break;
			case XVShapeStyle:
				strcpy(resource_values[i].r_value, buttonshapeinfo[objects[thisobj].shape]);
				break;
			case XVJustify:
				strcpy(resource_values[i].r_value, justifyinfo[objects[thisobj].justify]);
				break;
			default:
				strcpy(resource_values[i].r_value, "Error!");
				break;
		}
	}
}

// ----------------------------------------------

// *****************
// setNewResourceValue moves a new value into the current object in focus
// and re-reads the object's resources in case any changes have been made by
// Xaw
//
void setNewResourceValue(Boolean visual)
{
	int otype=objects[prop_object_index].object_type;
	Boolean  jerky=FALSE;

	objects[objects[prop_object_index].parent_window].changed=TRUE;
	if (strcmp(resource_values[selected_index].r_restype, XtNborderWidth) == 0) 
		if (otype != XVWindow) {
			objects[prop_object_index].pcore->Hide();
			jerky=TRUE;
		}
	else
		objects[prop_object_index].pcore->Unmap();
		
	setResourceExec(visual);

	if (jerky)
		objects[prop_object_index].pcore->Show();
	else
		objects[prop_object_index].pcore->Map();
}

char *getMapName(String src, String dest)
{
	int l=strlen(src)-1;
	char c;
	int ptr=0, j;

	c=src[l];
	while ((c!='/') && (l>=0)) {
		ptr++;
		l--;
		c=src[l];
	}
	l++;
	for (j=l; j<strlen(src); j++)
		dest[j-l]=src[j];

	dest[j-l]='\0';
	char *s=strchr(dest, '.');
	if (s!=NULL) *s='\0';	
	return(dest);
}

void setResourceExec(Boolean visual)
{
	char tempbuffer[50];
	char temp2buffer[50];
	int temptime;
	objectType pt;
	int pixtype=XVBITMAP;
	Pixel pix;

	if (ObjectBeingSized != -1) {
		ObjectBeingSized=-1;
		removeSizer();
	}
	
	if (ColoursVisible) {
		xvisColours.Close();
		ColoursVisible=TRUE;
	}

	int ok=0;
	int otype=objects[prop_object_index].object_type;

	strcpy(resource_values[selected_index].r_value, propeditbuffer);
	switch(resource_values[selected_index].r_type) {
		case XVPixel:
			if (strcmp(resource_values[selected_index].r_restype, XtNbackground)==0) {
				sprintf(objects[prop_object_index].background_colour, "%s", 
					resource_values[selected_index].r_value);
			}
			if (strcmp(resource_values[selected_index].r_restype, XtNforeground)==0) {
				sprintf(objects[prop_object_index].foreground_colour, "%s", 
					resource_values[selected_index].r_value);
			}
			if (strcmp(resource_values[selected_index].r_restype, XtNborderColor)==0) {
				sprintf(objects[prop_object_index].border_colour, "%s", 
					resource_values[selected_index].r_value);
			}
			if (strcmp(resource_values[selected_index].r_restype, XtNtopShadowPixel)==0) {
				sprintf(objects[prop_object_index].topshadow_colour, "%s", 
					resource_values[selected_index].r_value);
			}
			if (strcmp(resource_values[selected_index].r_restype, XtNbottomShadowPixel)==0) {
				sprintf(objects[prop_object_index].bottomshadow_colour, "%s", 
					resource_values[selected_index].r_value);
			}					
			if (strcmp(resource_values[selected_index].r_value, "(default)")!=0) {
				pix=stringToPixel(*objects[prop_object_index].pcore, 
					resource_values[selected_index].r_value);
				objects[prop_object_index].pcore->SetResource(
				resource_values[selected_index].r_restype, (XtArgVal) pix);
				if (otype==XVWindow) {
					if (strcmp(resource_values[selected_index].r_restype, XtNbackground)==0) 
						objects[prop_object_index].pwindow->GCSetBackground(pix);		
					
					if (strcmp(resource_values[selected_index].r_restype, XtNforeground)==0) 
						objects[prop_object_index].pwindow->GCSetForeground(pix);		

					drawGrid(prop_object_index);
				}
			}
			break;
		case XVDimension:
			sscanf(resource_values[selected_index].r_value, "%d", 
				&resource_values[selected_index].r_dimension);

			switch(otype) {
				case XVList:
					objects[prop_object_index].plist->xvView::SetResource(
						resource_values[selected_index].r_restype, 
						(XtArgVal) resource_values[selected_index].r_dimension);
					break;
				case XVFileList:
					objects[prop_object_index].pfilelist->xvView::SetResource(
						resource_values[selected_index].r_restype, 
						(XtArgVal) resource_values[selected_index].r_dimension);
					break;
				case XVDirList:
					objects[prop_object_index].pdirlist->xvView::SetResource(
						resource_values[selected_index].r_restype, 
						(XtArgVal) resource_values[selected_index].r_dimension);
					break;
				case XVDropDown:
					if (strcmp(resource_values[selected_index].r_restype, XtNheight)==0)
						resource_values[selected_index].r_dimension=24;
				default:
					objects[prop_object_index].pcore->SetResource(
						resource_values[selected_index].r_restype, 
						(XtArgVal) resource_values[selected_index].r_dimension);
					break;
			}
			break;
		case XVPosition:
			sscanf(resource_values[selected_index].r_value, "%d", 
				&resource_values[selected_index].r_position);
			switch(otype) {
				case XVList:
					objects[prop_object_index].plist->xvView::SetResource(
						resource_values[selected_index].r_restype, 
						(XtArgVal) resource_values[selected_index].r_position);
					break;
				case XVFileList:
					objects[prop_object_index].pfilelist->xvView::SetResource(
						resource_values[selected_index].r_restype, 
						(XtArgVal) resource_values[selected_index].r_position);
					break;
				case XVDirList:
					objects[prop_object_index].pdirlist->xvView::SetResource(
						resource_values[selected_index].r_restype, 
						(XtArgVal) resource_values[selected_index].r_position);
					break;
				default:
					objects[prop_object_index].pcore->SetResource(
						resource_values[selected_index].r_restype, 
						(XtArgVal) resource_values[selected_index].r_position);
					break;
			}
			break;
		case XVBoolean:
			break;
		case XVLocBoolean: 
			if (strcmp(resource_values[selected_index].r_restype, xvNfixed)==0) {
				if (strcmp(resource_values[selected_index].r_value, "TRUE")==0) {
					objects[prop_object_index].fixed_size=TRUE;
					objects[prop_object_index].pwindow->FixBorder(
					objects[prop_object_index].winwidth,
					objects[prop_object_index].winheight);
				}
				else {
					objects[prop_object_index].fixed_size=FALSE;
					objects[prop_object_index].pwindow->ReleaseBorder();
					
				}
			}
			if (strcmp(resource_values[selected_index].r_restype, xvNiconic)==0) {
				if (strcmp(resource_values[selected_index].r_value, "TRUE")==0) 
					objects[prop_object_index].iconic=TRUE;
				else
					objects[prop_object_index].iconic=FALSE;
			} 
			if (strcmp(resource_values[selected_index].r_restype, xvNopenAtStart)==0) {
				if (strcmp(resource_values[selected_index].r_value, "TRUE")==0) 
					objects[prop_object_index].open_at_startup=TRUE;
				else
					objects[prop_object_index].open_at_startup=FALSE;
			} 
			if (strcmp(resource_values[selected_index].r_restype, xvNenabled)==0) {
				if (strcmp(resource_values[selected_index].r_value, "TRUE")==0) 
					objects[prop_object_index].enabled=TRUE;
				else
					objects[prop_object_index].enabled=FALSE;
			} 
			if (strcmp(resource_values[selected_index].r_restype, xvNvisible)==0) {
				if (strcmp(resource_values[selected_index].r_value, "TRUE")==0) 
					objects[prop_object_index].visible=TRUE;
				else
					objects[prop_object_index].visible=FALSE;
			} 
			if (strcmp(resource_values[selected_index].r_restype, xvNautowrap)==0) {
				if (strcmp(resource_values[selected_index].r_value, "TRUE")==0) 
					objects[prop_object_index].autowrap=TRUE;
				else
					objects[prop_object_index].autowrap=FALSE;
			} 
			if (strcmp(resource_values[selected_index].r_restype, xvNctrl)==0) {
				if (strcmp(resource_values[selected_index].r_value, "TRUE")==0) 
					objects[prop_object_index].ctrl=TRUE;
				else
					objects[prop_object_index].ctrl=FALSE;
			} 
			if (strcmp(resource_values[selected_index].r_restype, xvNecho)==0) {
				if (strcmp(resource_values[selected_index].r_value, "TRUE")==0) 
					objects[prop_object_index].echo=TRUE;
				else
					objects[prop_object_index].echo=FALSE;
			} 
			break;
		case XVName:
			sprintf(tempbuffer, "Object %s already exists!!!", propeditbuffer);
			if ((visual) && (objectExists(propeditbuffer))) {
				alert.init("Object Exists", xvisProps, tempbuffer, XVMSGSIMPLEFATAL, "SkyBlue",
					alert_call, alert_call);
				alert.Open((int)screenWidth/2, (int)screenHeight/2);
			}
			else {
				strcpy(tempbuffer, objects[prop_object_index].name);
				strcpy(objects[prop_object_index].name, resource_values[selected_index].r_value);
				strcpy(temp2buffer, objects[prop_object_index].name);
				updateProjectListWindow();
				if (visual) updateCallbackCode(tempbuffer, temp2buffer);
			}
			break;
		case XVInt:
			break;
		case XVString:
			resource_values[selected_index].r_string=resource_values[selected_index].r_value;
			objects[prop_object_index].pcore->SetResource(resource_values[selected_index].r_restype, 
				(XtArgVal) resource_values[selected_index].r_string);
			break;
		case XVCursor:
			if (strcmp(propeditbuffer, "(None)")!=0) {
				resource_values[selected_index].r_cursor=resource_values[selected_index].r_value;
				objects[prop_object_index].pcore->SetCursorName(NULL);
				objects[prop_object_index].pcore->SetCursorName(
					resource_values[selected_index].r_cursor);
			}
			strcpy(objects[prop_object_index].obj_cursor, resource_values[selected_index].r_value);
			break;
		case XVWinName:
			resource_values[selected_index].r_string=resource_values[selected_index].r_value;
			objects[prop_object_index].pwindow->SetWindowResource(resource_values[selected_index].r_restype, 
				(XtArgVal) resource_values[selected_index].r_string);
			objects[prop_object_index].pwindow->SetWindowResource(XtNiconName, 
				(XtArgVal) resource_values[selected_index].r_string);
			break;
		case XVWinSize:
				sscanf(resource_values[selected_index].r_value, "%d", 
				&resource_values[selected_index].r_dimension);
			objects[prop_object_index].pwindow->SetWindowResource(resource_values[selected_index].r_restype,
 				(XtArgVal) resource_values[selected_index].r_dimension);
			objects[prop_object_index].pwindow->SetResource(resource_values[selected_index].r_restype,
 				(XtArgVal) resource_values[selected_index].r_dimension);
			if (strcmp(resource_values[selected_index].r_restype, XtNwidth)==0) 
				objects[prop_object_index].winwidth=resource_values[selected_index].r_dimension;
			if (strcmp(resource_values[selected_index].r_restype, XtNheight)==0)
				objects[prop_object_index].winheight=resource_values[selected_index].r_dimension;
			break;
		case XVWinPositionX:
		case XVWinPositionY:
			sscanf(resource_values[selected_index].r_value, "%d", 
				&resource_values[selected_index].r_position);
			objects[prop_object_index].pwindow->SetWindowResource(resource_values[selected_index].r_restype, 
				(XtArgVal) resource_values[selected_index].r_position);
			if (resource_values[selected_index].r_type==XVWinPositionX) {
				objects[prop_object_index].winxpos=resource_values[selected_index].r_position;
			}
			else {
				objects[prop_object_index].winypos=resource_values[selected_index].r_position;
			}	
			sprintf(tempbuffer, "+%d+%d", objects[prop_object_index].winxpos,
				objects[prop_object_index].winypos);
			objects[prop_object_index].pwindow->SetWindowResource(XtNgeometry, 
				(XtArgVal) resource_values[selected_index].r_position);			
			break;
		case XVFont:
			strcpy(objects[prop_object_index].obj_font, resource_values[selected_index].r_value);
			objects[prop_object_index].object_font->FLoadFont(
				objects[prop_object_index].obj_font);
			objects[prop_object_index].pcore->SetResource(XtNfont, 
				(XtArgVal)objects[prop_object_index].object_font->FontRef());
			break;			
		case XVPath:
			strcpy(objects[prop_object_index].obj_path, resource_values[selected_index].r_value);
			switch(otype) {
				case XVFileList:			
					objects[prop_object_index].pfilelist->SetNewPath(
						objects[prop_object_index].obj_path, XVFILELIST, FALSE);
					break;
				case XVDirList:			
					objects[prop_object_index].pfilelist->SetNewPath(
						objects[prop_object_index].obj_path, XVDIRLIST, FALSE);
					break;
				default:
					break;
			}	
			break;
		case XVBitmapFile:
			if (strcmp(resource_values[selected_index].r_value, "(None)")!=0) {

				pixtype=XVBITMAP;

				if (strstr(resource_values[selected_index].r_value, ".xpm")!=NULL)
					pixtype=XVPIXMAP;				

				if ( (strcmp(resource_values[selected_index].r_restype, xvNbitmapFile)==0) || (strcmp(resource_values[selected_index].r_restype, xvNicon)==0) )
					switch(otype) {
						case XVBitmap:
							ok=objects[prop_object_index].pbitmap->ChangeBitmap(
								resource_values[selected_index].r_value, pixtype);
							break;
						default:
							ok=objects[prop_object_index].object_bitmap->ReadFile(
								resource_values[selected_index].r_value, pixtype);
							break;
					} 
				else
					ok=objects[prop_object_index].leftbit->ReadFile(
						resource_values[selected_index].r_value, pixtype);

				if (ok==0) {
					if ( (strcmp(resource_values[selected_index].r_restype, xvNbitmapFile)==0) || (strcmp(resource_values[selected_index].r_restype, xvNicon)==0) )
						switch(otype) {
							case XVWindow:
								strcpy(objects[prop_object_index].obj_icon, 
									resource_values[selected_index].r_value);
								objects[prop_object_index].pwindow->SetWindowResource(XtNiconPixmap, 
									(XtArgVal)objects[prop_object_index].object_bitmap->GetPixmap());
								strcpy(objects[prop_object_index].obj_bitref, 
									getMapName(objects[prop_object_index].obj_icon, tempbuffer));
								objects[prop_object_index].has_bitmap=TRUE;
								break;
							case XVBitmap:
								strcpy(objects[prop_object_index].obj_path, 
									resource_values[selected_index].r_value);
								strcpy(objects[prop_object_index].obj_bitref, 
									getMapName(objects[prop_object_index].obj_path, tempbuffer));
								objects[prop_object_index].has_bitmap=TRUE;
								break;
							default:
								strcpy(objects[prop_object_index].obj_path, 
									resource_values[selected_index].r_value);
								objects[prop_object_index].pcore->SetResource(XtNbitmap, 
									(XtArgVal)objects[prop_object_index].object_bitmap->GetPixmap());
								strcpy(objects[prop_object_index].obj_bitref, 
									getMapName(objects[prop_object_index].obj_path, tempbuffer));
								objects[prop_object_index].has_bitmap=TRUE;
								break;
						}
					else {
						strcpy(objects[prop_object_index].leftbitmap, 
							resource_values[selected_index].r_value);
						objects[prop_object_index].pcore->SetResource(XtNleftBitmap, 
							(XtArgVal)objects[prop_object_index].leftbit->GetPixmap());
						strcpy(objects[prop_object_index].obj_leftbitref, 
							getMapName(objects[prop_object_index].leftbitmap, tempbuffer));
						objects[prop_object_index].has_leftbitmap=TRUE;
					}
				}						
				else
					if ( (strcmp(resource_values[selected_index].r_restype, xvNbitmapFile)==0) || (strcmp(resource_values[selected_index].r_restype, xvNicon)==0) )
						switch(otype) {
							case XVWindow:
								strcpy(resource_values[selected_index].r_value, 
									objects[prop_object_index].obj_icon);
								break;
							default:
								strcpy(resource_values[selected_index].r_value, 
									objects[prop_object_index].obj_path);
								break;
						}
					else
						strcpy(resource_values[selected_index].r_value, 
							objects[prop_object_index].leftbitmap);
			}
			else 
				if (maptype != 0) 
					switch (maptype) {
						case 1:
							objects[prop_object_index].has_bitmap=FALSE;
							strcpy(objects[prop_object_index].obj_bitref, "(None)");
							strcpy(objects[prop_object_index].obj_path, "(None)");				
							objects[prop_object_index].pcore->SetResource(XtNbitmap, 
								(XtArgVal) None);
							break;
						case 2: 
							objects[prop_object_index].has_leftbitmap=FALSE;
							strcpy(objects[prop_object_index].obj_leftbitref, "(None)");
							strcpy(objects[prop_object_index].leftbitmap, "(None)");
							strcpy(objects[prop_object_index].obj_path, "(None)");				
							objects[prop_object_index].pcore->SetResource(XtNleftBitmap, 
								(XtArgVal) None);
							break;
						case 3:
							objects[prop_object_index].has_bitmap=FALSE;
							strcpy(objects[prop_object_index].obj_icon, "(None)");
							objects[prop_object_index].pwindow->SetWindowResource(XtNiconPixmap, 
								(XtArgVal) None);
							break;
						default:
							break;
					}
				
			break;
		case XVObject:
			if (strcmp(resource_values[selected_index].r_value, "(None)")!=0) {
				objects[prop_object_index].object_ref3=selected_ref;
				objects[prop_object_index].ptoggle->ChangeGroup(
					*objects[selected_ref].pcore);
 			}
			else {
				objects[prop_object_index].object_ref3=-1;
				XawToggleChangeRadioGroup(objects[prop_object_index].ptoggle->ID(), NULL);				
			}
			break;
		case XVTime:
			sscanf(resource_values[selected_index].r_value, "%d", &temptime);
			objects[prop_object_index].object_time=temptime;			
			break;
		case XVShapeStyle:
			objects[prop_object_index].pcore->SetResource(XtNshapeStyle, 
				(XtArgVal) buttonshapecmd[objects[prop_object_index].shape]);
			break;
		case XVJustify:
			objects[prop_object_index].pcore->SetResource(XtNjustify, 
				(XtArgVal) justifycmd[objects[prop_object_index].justify]);
			break;
		case XVResize:
		case XVvScroll:
		case XVhScroll:
		case XVWrap:
		case XVEditmode:
			break;
		case XVVoid:
		default:
			break;
	}

	if (visual) {
		recoverResources(object_resource_number[prop_object_type], prop_object_index);		
		for (int i=0; i<object_resource_number[prop_object_type]; i++)
			resvallist[i]=resource_values[i].r_value;

		SetupPropsWindow(prop_object_type, resvallist, FALSE, prop_object_index);
		valuelist.Highlight(selected_index);
		proplist.Highlight(selected_index);
	}

}

// ----------------------------------------------


Boolean objectExists(String newname)
{
	for (int i=0; i<MAXOBJECTS; i++)
		if (objects[i].object_type != XVNone)
			if (strcmp(objects[i].name, newname)==0)
				return(TRUE);

	return(FALSE);
}

// ----------------------------------------------
